/*
 * A simple widget that is just a foreground-coloured rectangle.
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "BandP.h"

/*
 * Class Methods
 */
static void BandInitialize(Widget, Widget, ArgList, Cardinal*);
static void BandExpose(Widget, XEvent *, Region);
static void BandDestroy(Widget);
static Boolean BandSetValues(Widget, Widget, Widget, ArgList, Cardinal*);

static GC MakeGC(BandWidget, Pixel);

#define offset(field) XtOffsetOf(BandRec, band.field)
static XtResource resources[] = {
	{ XtNforeground, XtCForeground, XtRPixel,
	  sizeof (Pixel), offset(foreground),
	  XtRString, (XtPointer)XtDefaultForeground },
	{ XtNpercent, XtCPercent, XtRInt,
	  sizeof (int), offset(percent),
	  XtRImmediate, (XtPointer)100 }
};
#undef offset

#define Superclass	(&simpleClassRec)
BandClassRec bandClassRec = {
  /* core */
  {
    (WidgetClass)Superclass,		/* superclass */
    "Band",				/* class_name */
    sizeof(BandRec),			/* widget_size */
    NULL,				/* class_initialize */
    NULL,				/* class_part_initialize */
    False,				/* class_inited */
    BandInitialize,			/* initialize */
    NULL,				/* initialize_hook */
    XtInheritRealize,			/* realize */
    NULL,				/* actions */
    0,					/* num_actions */
    resources,				/* resources */
    XtNumber(resources),		/* num_resources */
    NULLQUARK,				/* xrm_class */
    True,				/* compress_motion */
    True,				/* compress_exposure */
    True,				/* compress_enterleave */
    False,				/* visible_interest */
    BandDestroy,			/* destroy */
    NULL,				/* resize */
    BandExpose,				/* expose */
    BandSetValues,			/* set_values */
    NULL,				/* set_values_hook */
    XtInheritSetValuesAlmost,		/* set_values_almost */
    NULL,				/* get_values_hook */
    NULL,				/* accept_focus */
    XtVersion,				/* version */
    NULL,				/* callback_private */
    NULL,				/* tm_table */
    XtInheritQueryGeometry,		/* query_geometry */
    XtInheritDisplayAccelerator,	/* display_accelerator */
    NULL,				/* extension */
  },
  /* simple */
  {
    XtInheritChangeSensitive,		/* change_sensitive */
  },
  /* band */
  {
    NULL,				/* extension */
  }
};

WidgetClass bandWidgetClass = (WidgetClass)&bandClassRec;

static void
BandExpose(Widget w, XEvent *ev, Region r)
{
    BandWidget bw = (BandWidget)w;
    unsigned int width = bw->core.width * bw->band.percent / 100;

    XFillRectangle(XtDisplay(w), XtWindow(w), bw->band.gc, width/2, 0, 
	bw->core.width - width/2, bw->core.height);
}

static GC
MakeGC(BandWidget bw, Pixel pixel)
{
    XGCValues values;

    values.foreground = pixel;
    return XtGetGC((Widget)bw, GCForeground, &values);
}

static void
BandInitialize(Widget request, Widget w, ArgList args, Cardinal *num_args)
{
    BandWidget bw = (BandWidget)w;

    bw->band.gc = MakeGC(bw, bw->band.foreground);
}

static void
BandDestroy(Widget w)
{
    BandWidget bw = (BandWidget)w;

    XtReleaseGC((Widget)bw, bw->band.gc);
}

static Boolean
BandSetValues(Widget current, Widget request, Widget new,
    ArgList args, Cardinal *num_args)
{
    BandWidget currentbw = (BandWidget)current;
    BandWidget newbw = (BandWidget)new;
    Boolean need_redraw = FALSE;

    if (currentbw->band.foreground != newbw->band.foreground) {
	XtReleaseGC((Widget)newbw, newbw->band.gc);
	newbw->band.gc = MakeGC(newbw, newbw->band.foreground);
	need_redraw = TRUE;
    }

    if (newbw->band.percent < 0)
        newbw->band.percent = 0;
    if (newbw->band.percent > 100)
        newbw->band.percent = 100;
    if (currentbw->band.percent != newbw->band.percent)
	need_redraw = TRUE;

    return need_redraw;
}

