#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Sme.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xmu/Editres.h>
#include "Band.h"

#define PLUSMINUS	"\xb1"
#define TIMES		"\xd7"

static void Quit(Widget w, XEvent *e, String *params, Cardinal *num_params);
static void Select(Widget w, XtPointer closure, XtPointer call_data);
static void ComputeResistorValue(void);

static String fallback_resources[] = {
	"?.width: 160",
	"?.height: 210",
	"?.allowShellResize: true",
	"*.saveUnder: true",
	NULL
};

static XrmOptionDescRec options[] = {
        /* { "-flag", ".hiertree.flag", XrmoptionNoArg, (XtPointer)"true" }, */
	/* { "-option", ".hiertree.option", XrmoptionSepArg }, */
};

static XtActionsRec actions[] = {
        { "quit",       Quit }
};

struct menuop {
	Cardinal op;
	Cardinal value;
	Widget menu;
};
Cardinal setting[5];

static XtAppContext appContext;
static Widget toplevel, result, band[5];

int
main(argc, argv)
	int argc;
	char *argv[];
{
	Display *display;
	Atom wm_delete_window;
	Cardinal i, j;
	/*Cardinal nargs; Arg args[10]; */
	Widget paned, menubar, bandbox, menu;
        Widget menubutton, bsb;
	struct menuop *op;

	toplevel = XtOpenApplication(
		&appContext,
		"Resistor",
		options, XtNumber(options),
		&argc, argv,
		fallback_resources,
		sessionShellWidgetClass,
		NULL, 0
	);

	XtAppAddActions(appContext, actions, XtNumber(actions));

	paned = XtCreateManagedWidget("paned", panedWidgetClass, 
	    toplevel, NULL, 0);

	/* Resistance result */
	result = XtCreateManagedWidget("result", labelWidgetClass,
	    paned, NULL, 0);

	/* Band box */
	bandbox = XtCreateManagedWidget("bandbox", boxWidgetClass,
	    paned, NULL, 0);
	for (j = 0; j < 5; j++)
	    band[j] = XtCreateManagedWidget("band", 
		bandWidgetClass, bandbox, NULL, 0);

	/* Menu bar */
	menubar = XtCreateManagedWidget("menubar", boxWidgetClass, 
	    paned, NULL, 0);

	/* Digits menu */
	for (j = 0; j < 5; j++) {
	    menubutton = XtCreateManagedWidget(
		j == 3 ? "multiplier" :
		j == 4 ? "tolerance" :
	                 "digit", menuButtonWidgetClass,
	        menubar, NULL, 0);
	    menu = XtCreatePopupShell("menu", simpleMenuWidgetClass,
		menubutton, NULL, 0);
	    for (i = 0; i <= 9; i++) {
	        char name[2] = { '0' + i, '\0' };

		if (j == 4 && (i >= 3 && i <= 6)) continue;
		if ((j == 3 || j == 4) && i == 7) {
		    XtCreateManagedWidget(name, smeLineObjectClass,
		        menu, NULL, 0);
		    continue;
		}
		op = XtNew(struct menuop);
		op->op = j;
		op->value = i;
		op->menu = menubutton;
		bsb = XtCreateManagedWidget(name, smeBSBObjectClass, 
		    menu, NULL, 0);
	        XtAddCallback(bsb, XtNcallback, Select, op);
		if (i == 0)
			Select(bsb, op, NULL);
	    }
	}

	XtRealizeWidget(toplevel);

	/* Translate window manager close request into quit */
	display = XtDisplay(toplevel);
	wm_delete_window = XInternAtom(display, "WM_DELETE_WINDOW", False);
	XSetWMProtocols(display, XtWindow(toplevel), &wm_delete_window, 1);
	XtOverrideTranslations(toplevel,
	    XtParseTranslationTable("<Message>WM_PROTOCOLS:quit()"));

	/* Enable editres(1) */
	XtAddEventHandler(toplevel, 0, True, _XEditResCheckMessages, NULL);

	XtAppMainLoop(appContext);
	exit(0);
}

static void
Quit(Widget w, XEvent *e, String *params, Cardinal *num_params)
{
	/* XtDestroyWidget(toplevel); */
	XtAppSetExitFlag(appContext);
}

/* Called when a menu item is selected */
static void
Select(Widget w, XtPointer closure, XtPointer call_data)
{
	Arg args[10];
	Cardinal argc;
	struct menuop *op = (struct menuop *)closure;
	Pixel fg;
	String label;
/*
	printf("op = %d, value = %d, menubutton = %s, w = %s\n",
	    op->op, op->value, XtName(op->menu), XtName(w));
*/
	setting[op->op] = op->value;

	/* Copy the label, foreground, and background values from
	 * the bsb to the menubutton */
	argc = 0;
	XtSetArg(args[argc], XtNforeground, &fg); argc++;
	XtSetArg(args[argc], XtNlabel, &label); argc++;
	XtGetValues(w, args, argc);

	argc = 0;
	XtSetArg(args[argc], XtNforeground, fg); argc++;
	XtSetArg(args[argc], XtNlabel, label); argc++;
	XtSetValues(op->menu, args, argc);

	argc = 0;
	XtSetArg(args[argc], XtNforeground, fg); argc++;
	XtSetValues(band[op->op], args, argc);

	ComputeResistorValue();
}

/* Computes the current value and sets the result label */
static void
ComputeResistorValue()
{
	char str[40];
	char *s, *tol, *label;
	Arg args[10];
	Cardinal argc;

	s = str;
	*s++ = '0' + setting[0];
	if (setting[3] == 1) *s++ = 'K';
	if (setting[3] == 4) *s++ = 'M';
	if (setting[3] == 9) *s++ = 'R';
	*s++ = '0' + setting[1];
	if (setting[3] == 2) *s++ = 'K';
	if (setting[3] == 5) *s++ = 'M';
	if (setting[3] == 8) *s++ = 'R';
	*s++ = '0' + setting[2];
	if (setting[3] == 3) *s++ = 'K';
	if (setting[3] == 6) *s++ = 'M';
	if (setting[3] == 0) *s++ = 'R';

	/* Remove trailing zeros */
	if (s[-1] == '0') s--;
	if (s[-1] == '0') s--;

	/* Add tolerance */
	switch (setting[4]) {
	case 0: /* tol = " " PLUSMINUS "20%"; break; */
	default: tol = ""; break;
	case 1: tol = " " PLUSMINUS "1%"; break;
	case 2: tol = " " PLUSMINUS "2%"; break;
	case 8: tol = " " PLUSMINUS "5%"; break;
	case 9: tol = " " PLUSMINUS "10%"; break;
	}
	while (*tol)
		*s++ = *tol++;

	*s = '\0';
	label = str;

	/* Remove leading zeros */
	if (label[0] == '0' && (label[1] >= '0' && label[1] <= '9')) label++;
	if (label[0] == '0' && (label[1] >= '0' && label[1] <= '9')) label++;

	argc = 0;
	XtSetArg(args[argc], XtNlabel, label); argc++;
	XtSetValues(result, args, argc);
}
